import * as helpers from './helpers/helpers.js';
import * as parse from './helpers/parse_responses.js';
import * as rankHelpers from './helpers/rank.js';
import * as simplify from './helpers/simplify.js';
import * as sitelinksHelpers from './helpers/sitelinks.js';
import * as timeHelpers from './helpers/time.js';
import { cirrusSearchPagesFactory } from './queries/cirrus_search.js';
import { getEntitiesFactory } from './queries/get_entities.js';
import { getEntitiesFromSitelinksFactory } from './queries/get_entities_from_sitelinks.js';
import { getEntityRevisionFactory } from './queries/get_entity_revision.js';
import { getManyEntitiesFactory } from './queries/get_many_entities.js';
import { getReverseClaimsFactory } from './queries/get_reverse_claims.js';
import { getRevisionsFactory } from './queries/get_revisions.js';
import { searchEntitiesFactory } from './queries/search_entities.js';
import { sparqlQueryFactory } from './queries/sparql_query.js';
import { type Url } from './utils/build_url.js';
import type { InstanceConfig } from './types/options.js';
declare const common: {
    readonly wikibaseTimeToDateObject: typeof timeHelpers.wikibaseTimeToDateObject;
    readonly wikibaseTimeToEpochTime: (value: timeHelpers.TimeInputValue) => string | number;
    readonly wikibaseTimeToISOString: (value: timeHelpers.TimeInputValue) => string;
    readonly wikibaseTimeToSimpleDay: (value: timeHelpers.TimeInputValue) => string;
    readonly truthyPropertyClaims: typeof rankHelpers.truthyPropertyClaims;
    readonly nonDeprecatedPropertyClaims: typeof rankHelpers.nonDeprecatedPropertyClaims;
    readonly truthyClaims: typeof rankHelpers.truthyClaims;
    readonly getSitelinkUrl: typeof sitelinksHelpers.getSitelinkUrl;
    readonly getSitelinkData: typeof sitelinksHelpers.getSitelinkData;
    readonly isSite: (site: string) => site is "aawiki" | "aawikibooks" | "aawiktionary" | "abwiki" | "abwiktionary" | "acewiki" | "adywiki" | "afwiki" | "afwikibooks" | "afwikiquote" | "afwiktionary" | "akwiki" | "akwikibooks" | "akwiktionary" | "alswiki" | "alswikibooks" | "alswikiquote" | "alswiktionary" | "altwiki" | "amiwiki" | "amwiki" | "amwikiquote" | "amwiktionary" | "angwiki" | "angwikibooks" | "angwikiquote" | "angwikisource" | "angwiktionary" | "anpwiki" | "anwiki" | "anwiktionary" | "arcwiki" | "arwiki" | "arwikibooks" | "arwikinews" | "arwikiquote" | "arwikisource" | "arwikiversity" | "arwiktionary" | "arywiki" | "arzwiki" | "astwiki" | "astwikibooks" | "astwikiquote" | "astwiktionary" | "aswiki" | "aswikibooks" | "aswikiquote" | "aswikisource" | "aswiktionary" | "atjwiki" | "avkwiki" | "avwiki" | "avwiktionary" | "awawiki" | "aywiki" | "aywikibooks" | "aywiktionary" | "azbwiki" | "azwiki" | "azwikibooks" | "azwikiquote" | "azwikisource" | "azwiktionary" | "banwiki" | "banwikisource" | "barwiki" | "bat_smgwiki" | "bawiki" | "bawikibooks" | "bbcwiki" | "bclwiki" | "bclwikiquote" | "bclwiktionary" | "be_x_oldwiki" | "bewiki" | "bewikibooks" | "bewikiquote" | "bewikisource" | "bewiktionary" | "bgwiki" | "bgwikibooks" | "bgwikinews" | "bgwikiquote" | "bgwikisource" | "bgwiktionary" | "bhwiki" | "bhwiktionary" | "biwiki" | "biwikibooks" | "biwiktionary" | "bjnwiki" | "bjnwikiquote" | "bjnwiktionary" | "blkwiki" | "blkwiktionary" | "bmwiki" | "bmwikibooks" | "bmwikiquote" | "bmwiktionary" | "bnwiki" | "bnwikibooks" | "bnwikiquote" | "bnwikisource" | "bnwikivoyage" | "bnwiktionary" | "bowiki" | "bowikibooks" | "bowiktionary" | "bpywiki" | "brwiki" | "brwikiquote" | "brwikisource" | "brwiktionary" | "bswiki" | "bswikibooks" | "bswikinews" | "bswikiquote" | "bswikisource" | "bswiktionary" | "btmwiktionary" | "bugwiki" | "bxrwiki" | "cawiki" | "cawikibooks" | "cawikinews" | "cawikiquote" | "cawikisource" | "cawiktionary" | "cbk_zamwiki" | "cdowiki" | "cebwiki" | "cewiki" | "chowiki" | "chrwiki" | "chrwiktionary" | "chwiki" | "chwikibooks" | "chwiktionary" | "chywiki" | "ckbwiki" | "ckbwiktionary" | "commonswiki" | "cowiki" | "cowikibooks" | "cowikiquote" | "cowiktionary" | "crhwiki" | "crwiki" | "crwikiquote" | "crwiktionary" | "csbwiki" | "csbwiktionary" | "cswiki" | "cswikibooks" | "cswikinews" | "cswikiquote" | "cswikisource" | "cswikiversity" | "cswiktionary" | "cuwiki" | "cvwiki" | "cvwikibooks" | "cywiki" | "cywikibooks" | "cywikiquote" | "cywikisource" | "cywiktionary" | "dagwiki" | "dawiki" | "dawikibooks" | "dawikiquote" | "dawikisource" | "dawiktionary" | "dewiki" | "dewikibooks" | "dewikinews" | "dewikiquote" | "dewikisource" | "dewikiversity" | "dewikivoyage" | "dewiktionary" | "dgawiki" | "dinwiki" | "diqwiki" | "diqwiktionary" | "dsbwiki" | "dtywiki" | "dvwiki" | "dvwiktionary" | "dzwiki" | "dzwiktionary" | "eewiki" | "elwiki" | "elwikibooks" | "elwikinews" | "elwikiquote" | "elwikisource" | "elwikiversity" | "elwikivoyage" | "elwiktionary" | "emlwiki" | "enwiki" | "enwikibooks" | "enwikinews" | "enwikiquote" | "enwikisource" | "enwikiversity" | "enwikivoyage" | "enwiktionary" | "eowiki" | "eowikibooks" | "eowikinews" | "eowikiquote" | "eowikisource" | "eowikivoyage" | "eowiktionary" | "eswiki" | "eswikibooks" | "eswikinews" | "eswikiquote" | "eswikisource" | "eswikiversity" | "eswikivoyage" | "eswiktionary" | "etwiki" | "etwikibooks" | "etwikiquote" | "etwikisource" | "etwiktionary" | "euwiki" | "euwikibooks" | "euwikiquote" | "euwikisource" | "euwiktionary" | "extwiki" | "fatwiki" | "fawiki" | "fawikibooks" | "fawikinews" | "fawikiquote" | "fawikisource" | "fawikivoyage" | "fawiktionary" | "ffwiki" | "fiu_vrowiki" | "fiwiki" | "fiwikibooks" | "fiwikinews" | "fiwikiquote" | "fiwikisource" | "fiwikiversity" | "fiwikivoyage" | "fiwiktionary" | "fjwiki" | "fjwiktionary" | "fonwiki" | "foundationwiki" | "fowiki" | "fowikisource" | "fowiktionary" | "frpwiki" | "frrwiki" | "frwiki" | "frwikibooks" | "frwikinews" | "frwikiquote" | "frwikisource" | "frwikiversity" | "frwikivoyage" | "frwiktionary" | "furwiki" | "fywiki" | "fywikibooks" | "fywiktionary" | "gagwiki" | "ganwiki" | "gawiki" | "gawikibooks" | "gawikiquote" | "gawiktionary" | "gcrwiki" | "gdwiki" | "gdwiktionary" | "glkwiki" | "glwiki" | "glwikibooks" | "glwikiquote" | "glwikisource" | "glwiktionary" | "gnwiki" | "gnwikibooks" | "gnwiktionary" | "gomwiki" | "gomwiktionary" | "gorwiki" | "gorwiktionary" | "gotwiki" | "gotwikibooks" | "gpewiki" | "gucwiki" | "gurwiki" | "guwiki" | "guwikibooks" | "guwikiquote" | "guwikisource" | "guwiktionary" | "guwwiki" | "guwwikinews" | "guwwikiquote" | "guwwiktionary" | "gvwiki" | "gvwiktionary" | "hakwiki" | "hawiki" | "hawiktionary" | "hawwiki" | "hewiki" | "hewikibooks" | "hewikinews" | "hewikiquote" | "hewikisource" | "hewikivoyage" | "hewiktionary" | "hifwiki" | "hifwiktionary" | "hiwiki" | "hiwikibooks" | "hiwikiquote" | "hiwikisource" | "hiwikiversity" | "hiwikivoyage" | "hiwiktionary" | "howiki" | "hrwiki" | "hrwikibooks" | "hrwikiquote" | "hrwikisource" | "hrwiktionary" | "hsbwiki" | "hsbwiktionary" | "htwiki" | "htwikisource" | "huwiki" | "huwikibooks" | "huwikinews" | "huwikiquote" | "huwikisource" | "huwiktionary" | "hywiki" | "hywikibooks" | "hywikiquote" | "hywikisource" | "hywiktionary" | "hywwiki" | "hzwiki" | "iawiki" | "iawikibooks" | "iawiktionary" | "idwiki" | "idwikibooks" | "idwikiquote" | "idwikisource" | "idwiktionary" | "iewiki" | "iewikibooks" | "iewiktionary" | "igwiki" | "igwikiquote" | "igwiktionary" | "iiwiki" | "ikwiki" | "ikwiktionary" | "ilowiki" | "inhwiki" | "iowiki" | "iowiktionary" | "iswiki" | "iswikibooks" | "iswikiquote" | "iswikisource" | "iswiktionary" | "itwiki" | "itwikibooks" | "itwikinews" | "itwikiquote" | "itwikisource" | "itwikiversity" | "itwikivoyage" | "itwiktionary" | "iuwiki" | "iuwiktionary" | "jamwiki" | "jawiki" | "jawikibooks" | "jawikinews" | "jawikiquote" | "jawikisource" | "jawikiversity" | "jawikivoyage" | "jawiktionary" | "jbowiki" | "jbowiktionary" | "jvwiki" | "jvwikisource" | "jvwiktionary" | "kaawiki" | "kabwiki" | "kawiki" | "kawikibooks" | "kawikiquote" | "kawiktionary" | "kbdwiki" | "kbdwiktionary" | "kbpwiki" | "kcgwiki" | "kcgwiktionary" | "kgwiki" | "kiwiki" | "kjwiki" | "kkwiki" | "kkwikibooks" | "kkwikiquote" | "kkwiktionary" | "klwiki" | "klwiktionary" | "kmwiki" | "kmwikibooks" | "kmwiktionary" | "knwiki" | "knwikibooks" | "knwikiquote" | "knwikisource" | "knwiktionary" | "koiwiki" | "kowiki" | "kowikibooks" | "kowikinews" | "kowikiquote" | "kowikisource" | "kowikiversity" | "kowiktionary" | "krcwiki" | "krwiki" | "krwikiquote" | "kshwiki" | "kswiki" | "kswikibooks" | "kswikiquote" | "kswiktionary" | "kuwiki" | "kuwikibooks" | "kuwikiquote" | "kuwiktionary" | "kvwiki" | "kwwiki" | "kwwikiquote" | "kwwiktionary" | "kywiki" | "kywikibooks" | "kywikiquote" | "kywiktionary" | "ladwiki" | "lawiki" | "lawikibooks" | "lawikiquote" | "lawikisource" | "lawiktionary" | "lbewiki" | "lbwiki" | "lbwikibooks" | "lbwikiquote" | "lbwiktionary" | "lezwiki" | "lfnwiki" | "lgwiki" | "lijwiki" | "lijwikisource" | "liwiki" | "liwikibooks" | "liwikinews" | "liwikiquote" | "liwikisource" | "liwiktionary" | "lldwiki" | "lmowiki" | "lmowiktionary" | "lnwiki" | "lnwikibooks" | "lnwiktionary" | "lowiki" | "lowiktionary" | "lrcwiki" | "ltgwiki" | "ltwiki" | "ltwikibooks" | "ltwikiquote" | "ltwikisource" | "ltwiktionary" | "lvwiki" | "lvwikibooks" | "lvwiktionary" | "madwiki" | "maiwiki" | "map_bmswiki" | "mdfwiki" | "mediawikiwiki" | "metawiki" | "mgwiki" | "mgwikibooks" | "mgwiktionary" | "mhrwiki" | "mhwiki" | "mhwiktionary" | "minwiki" | "minwiktionary" | "miwiki" | "miwikibooks" | "miwiktionary" | "mkwiki" | "mkwikibooks" | "mkwikisource" | "mkwiktionary" | "mlwiki" | "mlwikibooks" | "mlwikiquote" | "mlwikisource" | "mlwiktionary" | "mniwiki" | "mniwiktionary" | "mnwiki" | "mnwikibooks" | "mnwiktionary" | "mnwwiki" | "mnwwiktionary" | "mowiki" | "mowiktionary" | "mrjwiki" | "mrwiki" | "mrwikibooks" | "mrwikiquote" | "mrwikisource" | "mrwiktionary" | "mswiki" | "mswikibooks" | "mswiktionary" | "mtwiki" | "mtwiktionary" | "muswiki" | "mwlwiki" | "myvwiki" | "mywiki" | "mywikibooks" | "mywiktionary" | "mznwiki" | "nahwiki" | "nahwikibooks" | "nahwiktionary" | "napwiki" | "napwikisource" | "nawiki" | "nawikibooks" | "nawikiquote" | "nawiktionary" | "nds_nlwiki" | "ndswiki" | "ndswikibooks" | "ndswikiquote" | "ndswiktionary" | "newiki" | "newikibooks" | "newiktionary" | "newwiki" | "ngwiki" | "niawiki" | "niawiktionary" | "nlwiki" | "nlwikibooks" | "nlwikinews" | "nlwikiquote" | "nlwikisource" | "nlwikivoyage" | "nlwiktionary" | "nnwiki" | "nnwikiquote" | "nnwiktionary" | "novwiki" | "nowiki" | "nowikibooks" | "nowikinews" | "nowikiquote" | "nowikisource" | "nowiktionary" | "nqowiki" | "nrmwiki" | "nsowiki" | "nvwiki" | "nywiki" | "ocwiki" | "ocwikibooks" | "ocwiktionary" | "olowiki" | "omwiki" | "omwiktionary" | "orwiki" | "orwikisource" | "orwiktionary" | "oswiki" | "outreachwiki" | "pagwiki" | "pamwiki" | "papwiki" | "pawiki" | "pawikibooks" | "pawikisource" | "pawiktionary" | "pcdwiki" | "pcmwiki" | "pdcwiki" | "pflwiki" | "pihwiki" | "piwiki" | "piwiktionary" | "plwiki" | "plwikibooks" | "plwikinews" | "plwikiquote" | "plwikisource" | "plwikivoyage" | "plwiktionary" | "pmswiki" | "pmswikisource" | "pnbwiki" | "pnbwiktionary" | "pntwiki" | "pswiki" | "pswikibooks" | "pswikivoyage" | "pswiktionary" | "ptwiki" | "ptwikibooks" | "ptwikinews" | "ptwikiquote" | "ptwikisource" | "ptwikiversity" | "ptwikivoyage" | "ptwiktionary" | "pwnwiki" | "quwiki" | "quwikibooks" | "quwikiquote" | "quwiktionary" | "rmwiki" | "rmwikibooks" | "rmwiktionary" | "rmywiki" | "rnwiki" | "rnwiktionary" | "roa_rupwiki" | "roa_rupwiktionary" | "roa_tarawiki" | "rowiki" | "rowikibooks" | "rowikinews" | "rowikiquote" | "rowikisource" | "rowikivoyage" | "rowiktionary" | "ruewiki" | "ruwiki" | "ruwikibooks" | "ruwikinews" | "ruwikiquote" | "ruwikisource" | "ruwikiversity" | "ruwikivoyage" | "ruwiktionary" | "rwwiki" | "rwwiktionary" | "sahwiki" | "sahwikiquote" | "sahwikisource" | "satwiki" | "sawiki" | "sawikibooks" | "sawikiquote" | "sawikisource" | "sawiktionary" | "scnwiki" | "scnwiktionary" | "scowiki" | "scwiki" | "scwiktionary" | "sdwiki" | "sdwikinews" | "sdwiktionary" | "sewiki" | "sewikibooks" | "sgwiki" | "sgwiktionary" | "shiwiki" | "shnwiki" | "shnwikibooks" | "shnwikivoyage" | "shnwiktionary" | "shwiki" | "shwiktionary" | "shywiktionary" | "simplewiki" | "simplewikibooks" | "simplewikiquote" | "simplewiktionary" | "siwiki" | "siwikibooks" | "siwiktionary" | "skrwiki" | "skrwiktionary" | "skwiki" | "skwikibooks" | "skwikiquote" | "skwikisource" | "skwiktionary" | "slwiki" | "slwikibooks" | "slwikiquote" | "slwikisource" | "slwikiversity" | "slwiktionary" | "smnwiki" | "smwiki" | "smwiktionary" | "snwiki" | "snwiktionary" | "sourceswiki" | "sowiki" | "sowiktionary" | "specieswiki" | "sqwiki" | "sqwikibooks" | "sqwikinews" | "sqwikiquote" | "sqwiktionary" | "srnwiki" | "srwiki" | "srwikibooks" | "srwikinews" | "srwikiquote" | "srwikisource" | "srwiktionary" | "sswiki" | "sswiktionary" | "stqwiki" | "stwiki" | "stwiktionary" | "suwiki" | "suwikibooks" | "suwikiquote" | "suwikisource" | "suwiktionary" | "svwiki" | "svwikibooks" | "svwikinews" | "svwikiquote" | "svwikisource" | "svwikiversity" | "svwikivoyage" | "svwiktionary" | "swwiki" | "swwikibooks" | "swwiktionary" | "szlwiki" | "szywiki" | "tawiki" | "tawikibooks" | "tawikinews" | "tawikiquote" | "tawikisource" | "tawiktionary" | "taywiki" | "tcywiki" | "tetwiki" | "tewiki" | "tewikibooks" | "tewikiquote" | "tewikisource" | "tewiktionary" | "tgwiki" | "tgwikibooks" | "tgwiktionary" | "thwiki" | "thwikibooks" | "thwikinews" | "thwikiquote" | "thwikisource" | "thwiktionary" | "tiwiki" | "tiwiktionary" | "tkwiki" | "tkwikibooks" | "tkwikiquote" | "tkwiktionary" | "tlwiki" | "tlwikibooks" | "tlwikiquote" | "tlwiktionary" | "tlywiki" | "tnwiki" | "tnwiktionary" | "towiki" | "towiktionary" | "tpiwiki" | "tpiwiktionary" | "trvwiki" | "trwiki" | "trwikibooks" | "trwikinews" | "trwikiquote" | "trwikisource" | "trwikivoyage" | "trwiktionary" | "tswiki" | "tswiktionary" | "ttwiki" | "ttwikibooks" | "ttwikiquote" | "ttwiktionary" | "tumwiki" | "twwiki" | "twwiktionary" | "tyvwiki" | "tywiki" | "udmwiki" | "ugwiki" | "ugwikibooks" | "ugwikiquote" | "ugwiktionary" | "ukwiki" | "ukwikibooks" | "ukwikinews" | "ukwikiquote" | "ukwikisource" | "ukwikivoyage" | "ukwiktionary" | "urwiki" | "urwikibooks" | "urwikiquote" | "urwiktionary" | "uzwiki" | "uzwikibooks" | "uzwikiquote" | "uzwiktionary" | "vecwiki" | "vecwikisource" | "vecwiktionary" | "vepwiki" | "vewiki" | "viwiki" | "viwikibooks" | "viwikiquote" | "viwikisource" | "viwikivoyage" | "viwiktionary" | "vlswiki" | "vowiki" | "vowikibooks" | "vowikiquote" | "vowiktionary" | "warwiki" | "wawiki" | "wawikibooks" | "wawikisource" | "wawiktionary" | "wikidatawiki" | "wikifunctionswiki" | "wikimaniawiki" | "wowiki" | "wowikiquote" | "wowiktionary" | "wuuwiki" | "xalwiki" | "xhwiki" | "xhwikibooks" | "xhwiktionary" | "xmfwiki" | "yiwiki" | "yiwikisource" | "yiwiktionary" | "yowiki" | "yowikibooks" | "yowiktionary" | "yuewiktionary" | "zawiki" | "zawikibooks" | "zawikiquote" | "zawiktionary" | "zeawiki" | "zghwiki" | "zh_classicalwiki" | "zh_min_nanwiki" | "zh_min_nanwikibooks" | "zh_min_nanwikiquote" | "zh_min_nanwikisource" | "zh_min_nanwiktionary" | "zh_yuewiki" | "zhwiki" | "zhwikibooks" | "zhwikinews" | "zhwikiquote" | "zhwikisource" | "zhwikiversity" | "zhwikivoyage" | "zhwiktionary" | "zuwiki" | "zuwikibooks" | "zuwiktionary";
    readonly isSitelinkKey: (site: string) => site is "aawiki" | "aawikibooks" | "aawiktionary" | "abwiki" | "abwiktionary" | "acewiki" | "adywiki" | "afwiki" | "afwikibooks" | "afwikiquote" | "afwiktionary" | "akwiki" | "akwikibooks" | "akwiktionary" | "alswiki" | "alswikibooks" | "alswikiquote" | "alswiktionary" | "altwiki" | "amiwiki" | "amwiki" | "amwikiquote" | "amwiktionary" | "angwiki" | "angwikibooks" | "angwikiquote" | "angwikisource" | "angwiktionary" | "anpwiki" | "anwiki" | "anwiktionary" | "arcwiki" | "arwiki" | "arwikibooks" | "arwikinews" | "arwikiquote" | "arwikisource" | "arwikiversity" | "arwiktionary" | "arywiki" | "arzwiki" | "astwiki" | "astwikibooks" | "astwikiquote" | "astwiktionary" | "aswiki" | "aswikibooks" | "aswikiquote" | "aswikisource" | "aswiktionary" | "atjwiki" | "avkwiki" | "avwiki" | "avwiktionary" | "awawiki" | "aywiki" | "aywikibooks" | "aywiktionary" | "azbwiki" | "azwiki" | "azwikibooks" | "azwikiquote" | "azwikisource" | "azwiktionary" | "banwiki" | "banwikisource" | "barwiki" | "bat_smgwiki" | "bawiki" | "bawikibooks" | "bbcwiki" | "bclwiki" | "bclwikiquote" | "bclwiktionary" | "be_x_oldwiki" | "bewiki" | "bewikibooks" | "bewikiquote" | "bewikisource" | "bewiktionary" | "bgwiki" | "bgwikibooks" | "bgwikinews" | "bgwikiquote" | "bgwikisource" | "bgwiktionary" | "bhwiki" | "bhwiktionary" | "biwiki" | "biwikibooks" | "biwiktionary" | "bjnwiki" | "bjnwikiquote" | "bjnwiktionary" | "blkwiki" | "blkwiktionary" | "bmwiki" | "bmwikibooks" | "bmwikiquote" | "bmwiktionary" | "bnwiki" | "bnwikibooks" | "bnwikiquote" | "bnwikisource" | "bnwikivoyage" | "bnwiktionary" | "bowiki" | "bowikibooks" | "bowiktionary" | "bpywiki" | "brwiki" | "brwikiquote" | "brwikisource" | "brwiktionary" | "bswiki" | "bswikibooks" | "bswikinews" | "bswikiquote" | "bswikisource" | "bswiktionary" | "btmwiktionary" | "bugwiki" | "bxrwiki" | "cawiki" | "cawikibooks" | "cawikinews" | "cawikiquote" | "cawikisource" | "cawiktionary" | "cbk_zamwiki" | "cdowiki" | "cebwiki" | "cewiki" | "chowiki" | "chrwiki" | "chrwiktionary" | "chwiki" | "chwikibooks" | "chwiktionary" | "chywiki" | "ckbwiki" | "ckbwiktionary" | "commonswiki" | "cowiki" | "cowikibooks" | "cowikiquote" | "cowiktionary" | "crhwiki" | "crwiki" | "crwikiquote" | "crwiktionary" | "csbwiki" | "csbwiktionary" | "cswiki" | "cswikibooks" | "cswikinews" | "cswikiquote" | "cswikisource" | "cswikiversity" | "cswiktionary" | "cuwiki" | "cvwiki" | "cvwikibooks" | "cywiki" | "cywikibooks" | "cywikiquote" | "cywikisource" | "cywiktionary" | "dagwiki" | "dawiki" | "dawikibooks" | "dawikiquote" | "dawikisource" | "dawiktionary" | "dewiki" | "dewikibooks" | "dewikinews" | "dewikiquote" | "dewikisource" | "dewikiversity" | "dewikivoyage" | "dewiktionary" | "dgawiki" | "dinwiki" | "diqwiki" | "diqwiktionary" | "dsbwiki" | "dtywiki" | "dvwiki" | "dvwiktionary" | "dzwiki" | "dzwiktionary" | "eewiki" | "elwiki" | "elwikibooks" | "elwikinews" | "elwikiquote" | "elwikisource" | "elwikiversity" | "elwikivoyage" | "elwiktionary" | "emlwiki" | "enwiki" | "enwikibooks" | "enwikinews" | "enwikiquote" | "enwikisource" | "enwikiversity" | "enwikivoyage" | "enwiktionary" | "eowiki" | "eowikibooks" | "eowikinews" | "eowikiquote" | "eowikisource" | "eowikivoyage" | "eowiktionary" | "eswiki" | "eswikibooks" | "eswikinews" | "eswikiquote" | "eswikisource" | "eswikiversity" | "eswikivoyage" | "eswiktionary" | "etwiki" | "etwikibooks" | "etwikiquote" | "etwikisource" | "etwiktionary" | "euwiki" | "euwikibooks" | "euwikiquote" | "euwikisource" | "euwiktionary" | "extwiki" | "fatwiki" | "fawiki" | "fawikibooks" | "fawikinews" | "fawikiquote" | "fawikisource" | "fawikivoyage" | "fawiktionary" | "ffwiki" | "fiu_vrowiki" | "fiwiki" | "fiwikibooks" | "fiwikinews" | "fiwikiquote" | "fiwikisource" | "fiwikiversity" | "fiwikivoyage" | "fiwiktionary" | "fjwiki" | "fjwiktionary" | "fonwiki" | "foundationwiki" | "fowiki" | "fowikisource" | "fowiktionary" | "frpwiki" | "frrwiki" | "frwiki" | "frwikibooks" | "frwikinews" | "frwikiquote" | "frwikisource" | "frwikiversity" | "frwikivoyage" | "frwiktionary" | "furwiki" | "fywiki" | "fywikibooks" | "fywiktionary" | "gagwiki" | "ganwiki" | "gawiki" | "gawikibooks" | "gawikiquote" | "gawiktionary" | "gcrwiki" | "gdwiki" | "gdwiktionary" | "glkwiki" | "glwiki" | "glwikibooks" | "glwikiquote" | "glwikisource" | "glwiktionary" | "gnwiki" | "gnwikibooks" | "gnwiktionary" | "gomwiki" | "gomwiktionary" | "gorwiki" | "gorwiktionary" | "gotwiki" | "gotwikibooks" | "gpewiki" | "gucwiki" | "gurwiki" | "guwiki" | "guwikibooks" | "guwikiquote" | "guwikisource" | "guwiktionary" | "guwwiki" | "guwwikinews" | "guwwikiquote" | "guwwiktionary" | "gvwiki" | "gvwiktionary" | "hakwiki" | "hawiki" | "hawiktionary" | "hawwiki" | "hewiki" | "hewikibooks" | "hewikinews" | "hewikiquote" | "hewikisource" | "hewikivoyage" | "hewiktionary" | "hifwiki" | "hifwiktionary" | "hiwiki" | "hiwikibooks" | "hiwikiquote" | "hiwikisource" | "hiwikiversity" | "hiwikivoyage" | "hiwiktionary" | "howiki" | "hrwiki" | "hrwikibooks" | "hrwikiquote" | "hrwikisource" | "hrwiktionary" | "hsbwiki" | "hsbwiktionary" | "htwiki" | "htwikisource" | "huwiki" | "huwikibooks" | "huwikinews" | "huwikiquote" | "huwikisource" | "huwiktionary" | "hywiki" | "hywikibooks" | "hywikiquote" | "hywikisource" | "hywiktionary" | "hywwiki" | "hzwiki" | "iawiki" | "iawikibooks" | "iawiktionary" | "idwiki" | "idwikibooks" | "idwikiquote" | "idwikisource" | "idwiktionary" | "iewiki" | "iewikibooks" | "iewiktionary" | "igwiki" | "igwikiquote" | "igwiktionary" | "iiwiki" | "ikwiki" | "ikwiktionary" | "ilowiki" | "inhwiki" | "iowiki" | "iowiktionary" | "iswiki" | "iswikibooks" | "iswikiquote" | "iswikisource" | "iswiktionary" | "itwiki" | "itwikibooks" | "itwikinews" | "itwikiquote" | "itwikisource" | "itwikiversity" | "itwikivoyage" | "itwiktionary" | "iuwiki" | "iuwiktionary" | "jamwiki" | "jawiki" | "jawikibooks" | "jawikinews" | "jawikiquote" | "jawikisource" | "jawikiversity" | "jawikivoyage" | "jawiktionary" | "jbowiki" | "jbowiktionary" | "jvwiki" | "jvwikisource" | "jvwiktionary" | "kaawiki" | "kabwiki" | "kawiki" | "kawikibooks" | "kawikiquote" | "kawiktionary" | "kbdwiki" | "kbdwiktionary" | "kbpwiki" | "kcgwiki" | "kcgwiktionary" | "kgwiki" | "kiwiki" | "kjwiki" | "kkwiki" | "kkwikibooks" | "kkwikiquote" | "kkwiktionary" | "klwiki" | "klwiktionary" | "kmwiki" | "kmwikibooks" | "kmwiktionary" | "knwiki" | "knwikibooks" | "knwikiquote" | "knwikisource" | "knwiktionary" | "koiwiki" | "kowiki" | "kowikibooks" | "kowikinews" | "kowikiquote" | "kowikisource" | "kowikiversity" | "kowiktionary" | "krcwiki" | "krwiki" | "krwikiquote" | "kshwiki" | "kswiki" | "kswikibooks" | "kswikiquote" | "kswiktionary" | "kuwiki" | "kuwikibooks" | "kuwikiquote" | "kuwiktionary" | "kvwiki" | "kwwiki" | "kwwikiquote" | "kwwiktionary" | "kywiki" | "kywikibooks" | "kywikiquote" | "kywiktionary" | "ladwiki" | "lawiki" | "lawikibooks" | "lawikiquote" | "lawikisource" | "lawiktionary" | "lbewiki" | "lbwiki" | "lbwikibooks" | "lbwikiquote" | "lbwiktionary" | "lezwiki" | "lfnwiki" | "lgwiki" | "lijwiki" | "lijwikisource" | "liwiki" | "liwikibooks" | "liwikinews" | "liwikiquote" | "liwikisource" | "liwiktionary" | "lldwiki" | "lmowiki" | "lmowiktionary" | "lnwiki" | "lnwikibooks" | "lnwiktionary" | "lowiki" | "lowiktionary" | "lrcwiki" | "ltgwiki" | "ltwiki" | "ltwikibooks" | "ltwikiquote" | "ltwikisource" | "ltwiktionary" | "lvwiki" | "lvwikibooks" | "lvwiktionary" | "madwiki" | "maiwiki" | "map_bmswiki" | "mdfwiki" | "mediawikiwiki" | "metawiki" | "mgwiki" | "mgwikibooks" | "mgwiktionary" | "mhrwiki" | "mhwiki" | "mhwiktionary" | "minwiki" | "minwiktionary" | "miwiki" | "miwikibooks" | "miwiktionary" | "mkwiki" | "mkwikibooks" | "mkwikisource" | "mkwiktionary" | "mlwiki" | "mlwikibooks" | "mlwikiquote" | "mlwikisource" | "mlwiktionary" | "mniwiki" | "mniwiktionary" | "mnwiki" | "mnwikibooks" | "mnwiktionary" | "mnwwiki" | "mnwwiktionary" | "mowiki" | "mowiktionary" | "mrjwiki" | "mrwiki" | "mrwikibooks" | "mrwikiquote" | "mrwikisource" | "mrwiktionary" | "mswiki" | "mswikibooks" | "mswiktionary" | "mtwiki" | "mtwiktionary" | "muswiki" | "mwlwiki" | "myvwiki" | "mywiki" | "mywikibooks" | "mywiktionary" | "mznwiki" | "nahwiki" | "nahwikibooks" | "nahwiktionary" | "napwiki" | "napwikisource" | "nawiki" | "nawikibooks" | "nawikiquote" | "nawiktionary" | "nds_nlwiki" | "ndswiki" | "ndswikibooks" | "ndswikiquote" | "ndswiktionary" | "newiki" | "newikibooks" | "newiktionary" | "newwiki" | "ngwiki" | "niawiki" | "niawiktionary" | "nlwiki" | "nlwikibooks" | "nlwikinews" | "nlwikiquote" | "nlwikisource" | "nlwikivoyage" | "nlwiktionary" | "nnwiki" | "nnwikiquote" | "nnwiktionary" | "novwiki" | "nowiki" | "nowikibooks" | "nowikinews" | "nowikiquote" | "nowikisource" | "nowiktionary" | "nqowiki" | "nrmwiki" | "nsowiki" | "nvwiki" | "nywiki" | "ocwiki" | "ocwikibooks" | "ocwiktionary" | "olowiki" | "omwiki" | "omwiktionary" | "orwiki" | "orwikisource" | "orwiktionary" | "oswiki" | "outreachwiki" | "pagwiki" | "pamwiki" | "papwiki" | "pawiki" | "pawikibooks" | "pawikisource" | "pawiktionary" | "pcdwiki" | "pcmwiki" | "pdcwiki" | "pflwiki" | "pihwiki" | "piwiki" | "piwiktionary" | "plwiki" | "plwikibooks" | "plwikinews" | "plwikiquote" | "plwikisource" | "plwikivoyage" | "plwiktionary" | "pmswiki" | "pmswikisource" | "pnbwiki" | "pnbwiktionary" | "pntwiki" | "pswiki" | "pswikibooks" | "pswikivoyage" | "pswiktionary" | "ptwiki" | "ptwikibooks" | "ptwikinews" | "ptwikiquote" | "ptwikisource" | "ptwikiversity" | "ptwikivoyage" | "ptwiktionary" | "pwnwiki" | "quwiki" | "quwikibooks" | "quwikiquote" | "quwiktionary" | "rmwiki" | "rmwikibooks" | "rmwiktionary" | "rmywiki" | "rnwiki" | "rnwiktionary" | "roa_rupwiki" | "roa_rupwiktionary" | "roa_tarawiki" | "rowiki" | "rowikibooks" | "rowikinews" | "rowikiquote" | "rowikisource" | "rowikivoyage" | "rowiktionary" | "ruewiki" | "ruwiki" | "ruwikibooks" | "ruwikinews" | "ruwikiquote" | "ruwikisource" | "ruwikiversity" | "ruwikivoyage" | "ruwiktionary" | "rwwiki" | "rwwiktionary" | "sahwiki" | "sahwikiquote" | "sahwikisource" | "satwiki" | "sawiki" | "sawikibooks" | "sawikiquote" | "sawikisource" | "sawiktionary" | "scnwiki" | "scnwiktionary" | "scowiki" | "scwiki" | "scwiktionary" | "sdwiki" | "sdwikinews" | "sdwiktionary" | "sewiki" | "sewikibooks" | "sgwiki" | "sgwiktionary" | "shiwiki" | "shnwiki" | "shnwikibooks" | "shnwikivoyage" | "shnwiktionary" | "shwiki" | "shwiktionary" | "shywiktionary" | "simplewiki" | "simplewikibooks" | "simplewikiquote" | "simplewiktionary" | "siwiki" | "siwikibooks" | "siwiktionary" | "skrwiki" | "skrwiktionary" | "skwiki" | "skwikibooks" | "skwikiquote" | "skwikisource" | "skwiktionary" | "slwiki" | "slwikibooks" | "slwikiquote" | "slwikisource" | "slwikiversity" | "slwiktionary" | "smnwiki" | "smwiki" | "smwiktionary" | "snwiki" | "snwiktionary" | "sourceswiki" | "sowiki" | "sowiktionary" | "specieswiki" | "sqwiki" | "sqwikibooks" | "sqwikinews" | "sqwikiquote" | "sqwiktionary" | "srnwiki" | "srwiki" | "srwikibooks" | "srwikinews" | "srwikiquote" | "srwikisource" | "srwiktionary" | "sswiki" | "sswiktionary" | "stqwiki" | "stwiki" | "stwiktionary" | "suwiki" | "suwikibooks" | "suwikiquote" | "suwikisource" | "suwiktionary" | "svwiki" | "svwikibooks" | "svwikinews" | "svwikiquote" | "svwikisource" | "svwikiversity" | "svwikivoyage" | "svwiktionary" | "swwiki" | "swwikibooks" | "swwiktionary" | "szlwiki" | "szywiki" | "tawiki" | "tawikibooks" | "tawikinews" | "tawikiquote" | "tawikisource" | "tawiktionary" | "taywiki" | "tcywiki" | "tetwiki" | "tewiki" | "tewikibooks" | "tewikiquote" | "tewikisource" | "tewiktionary" | "tgwiki" | "tgwikibooks" | "tgwiktionary" | "thwiki" | "thwikibooks" | "thwikinews" | "thwikiquote" | "thwikisource" | "thwiktionary" | "tiwiki" | "tiwiktionary" | "tkwiki" | "tkwikibooks" | "tkwikiquote" | "tkwiktionary" | "tlwiki" | "tlwikibooks" | "tlwikiquote" | "tlwiktionary" | "tlywiki" | "tnwiki" | "tnwiktionary" | "towiki" | "towiktionary" | "tpiwiki" | "tpiwiktionary" | "trvwiki" | "trwiki" | "trwikibooks" | "trwikinews" | "trwikiquote" | "trwikisource" | "trwikivoyage" | "trwiktionary" | "tswiki" | "tswiktionary" | "ttwiki" | "ttwikibooks" | "ttwikiquote" | "ttwiktionary" | "tumwiki" | "twwiki" | "twwiktionary" | "tyvwiki" | "tywiki" | "udmwiki" | "ugwiki" | "ugwikibooks" | "ugwikiquote" | "ugwiktionary" | "ukwiki" | "ukwikibooks" | "ukwikinews" | "ukwikiquote" | "ukwikisource" | "ukwikivoyage" | "ukwiktionary" | "urwiki" | "urwikibooks" | "urwikiquote" | "urwiktionary" | "uzwiki" | "uzwikibooks" | "uzwikiquote" | "uzwiktionary" | "vecwiki" | "vecwikisource" | "vecwiktionary" | "vepwiki" | "vewiki" | "viwiki" | "viwikibooks" | "viwikiquote" | "viwikisource" | "viwikivoyage" | "viwiktionary" | "vlswiki" | "vowiki" | "vowikibooks" | "vowikiquote" | "vowiktionary" | "warwiki" | "wawiki" | "wawikibooks" | "wawikisource" | "wawiktionary" | "wikidatawiki" | "wikifunctionswiki" | "wikimaniawiki" | "wowiki" | "wowikiquote" | "wowiktionary" | "wuuwiki" | "xalwiki" | "xhwiki" | "xhwikibooks" | "xhwiktionary" | "xmfwiki" | "yiwiki" | "yiwikisource" | "yiwiktionary" | "yowiki" | "yowikibooks" | "yowiktionary" | "yuewiktionary" | "zawiki" | "zawikibooks" | "zawikiquote" | "zawiktionary" | "zeawiki" | "zghwiki" | "zh_classicalwiki" | "zh_min_nanwiki" | "zh_min_nanwikibooks" | "zh_min_nanwikiquote" | "zh_min_nanwikisource" | "zh_min_nanwiktionary" | "zh_yuewiki" | "zhwiki" | "zhwikibooks" | "zhwikinews" | "zhwikiquote" | "zhwikisource" | "zhwikiversity" | "zhwikivoyage" | "zhwiktionary" | "zuwiki" | "zuwikibooks" | "zuwiktionary";
    readonly isPropertyClaimsId: typeof helpers.isPropertyClaimsId;
    readonly isEntityPageTitle: typeof helpers.isEntityPageTitle;
    readonly getNumericId: typeof helpers.getNumericId;
    readonly getImageUrl: typeof helpers.getImageUrl;
    readonly getEntityIdFromGuid: typeof helpers.getEntityIdFromGuid;
    readonly isNumericId: (id: string) => id is `${number}`;
    readonly isEntityId: (id: string) => id is import("./index.js").EntityId;
    readonly isEntitySchemaId: (id: string) => id is `E${number}`;
    readonly isItemId: (id: string) => id is `Q${number}`;
    readonly isPropertyId: (id: string) => id is `P${number}`;
    readonly isLexemeId: (id: string) => id is `L${number}`;
    readonly isFormId: (id: string) => id is `L${number}-F${number}`;
    readonly isSenseId: (id: string) => id is `L${number}-S${number}`;
    readonly isMediaInfoId: (id: string) => id is `M${number}`;
    readonly isGuid: (id: string) => id is `Q${number}$${string}` | `P${number}$${string}` | `L${number}$${string}` | `M${number}$${string}` | `L${number}-F${number}$${string}` | `L${number}-S${number}$${string}` | `q${Lowercase<`${number}`>}$${string}` | `p${Lowercase<`${number}`>}$${string}` | `l${Lowercase<`${number}`>}$${string}` | `m${Lowercase<`${number}`>}$${string}` | `l${Lowercase<`${number}`>}-f${Lowercase<`${number}`>}$${string}` | `l${Lowercase<`${number}`>}-s${Lowercase<`${number}`>}$${string}`;
    readonly isHash: (id: string) => id is string;
    readonly isRevisionId: (id: string) => id is `${number}`;
    readonly isNonNestedEntityId: (id: string) => id is import("./index.js").NonNestedEntityId;
    readonly simplify: typeof simplify;
    readonly parse: typeof parse;
};
type ApiQueries = {
    readonly searchEntities: ReturnType<typeof searchEntitiesFactory>;
    readonly cirrusSearchPages: ReturnType<typeof cirrusSearchPagesFactory>;
    readonly getEntities: ReturnType<typeof getEntitiesFactory>;
    readonly getManyEntities: ReturnType<typeof getManyEntitiesFactory>;
    readonly getRevisions: ReturnType<typeof getRevisionsFactory>;
    readonly getEntityRevision: ReturnType<typeof getEntityRevisionFactory>;
    readonly getEntitiesFromSitelinks: ReturnType<typeof getEntitiesFromSitelinksFactory>;
};
type SparqlQueries = {
    readonly sparqlQuery: ReturnType<typeof sparqlQueryFactory>;
    readonly getReverseClaims: ReturnType<typeof getReverseClaimsFactory>;
};
type Instance = {
    readonly root: Url;
    readonly apiEndpoint: Url;
};
export type Wbk = {
    readonly instance: Instance;
} & ApiQueries & SparqlQueries & typeof common;
export declare function WBK(config: InstanceConfig): Wbk;
export {};
//# sourceMappingURL=wikibase-sdk.d.ts.map